# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mathjson2qubo']

package_data = \
{'': ['*']}

install_requires = \
['pyqubo>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'mathjson2qubo',
    'version': '0.1.5',
    'description': 'mathjson2qubo allows you to convert MathJSON into PyQUBO model.',
    'long_description': '# mathjson2qubo\n\n[![PyPI version](https://badge.fury.io/py/mathjson2qubo.svg)](https://badge.fury.io/py/mathjson2qubo)\n![Package maintenance](https://github.com/mks1412/mathjson2qubo/workflows/Package%20maintenance/badge.svg)\n[![codecov](https://codecov.io/gh/mks1412/mathjson2qubo/branch/master/graph/badge.svg)](https://codecov.io/gh/mks1412/mathjson2qubo)\n\n> :warning: **This package is work in progress!**\n\n## About\n\nmathjson2qubo allows you to convert [MathJSON](https://cortexjs.io/guides/math-json/) into [PyQUBO](https://github.com/recruit-communications/pyqubo) expression.\n\nAssume that a Quadratic Unconstrained Binary Optimizationthe (QUBO) model as the following equation.\n\n<img src="https://latex.codecogs.com/gif.latex?\\sum^N_i\\sum^N_jv_{i,j}x_ix_j,~~x_i\\in\\{0,1\\}." title="QUBO equation" />\n\nIn MathJSON format, the above equation is formatted as the below json based on the latex abstract syntax tree (AST).\n\n```json\n{\n  "fn": "sum",\n  "sub": { "sym": "i" },\n  "sup": { "sym": "N" },\n  "arg": [\n    {\n      "fn": "sum",\n      "sub": { "sym": "j" },\n      "sup": { "sym": "N" },\n      "arg": [\n        {\n          "fn": "multiply",\n          "arg": [\n            {\n              "sym": "v",\n              "sub": {\n                "fn": "list",\n                "arg": [\n                  { "sym": "i" },\n                  { "sym": "j" }\n                ]\n              }\n            },\n            { "sym": "x", "sub": { "sym": "i" } },\n            { "sym": "x", "sub": { "sym": "j" } }\n          ]\n        }\n      ]\n    }\n  ]\n}\n```\n\nIn PyQUBO, the equation is expressed by the following code.\n\n```python\nfrom pyqubo import Array, Sum\n\n# Note that `v` in the equation is defined appropriately here.\n\nx = Array.create("x", N, "BINARY")\neq = Sum(0, N,\n    lambda i: Sum(0, N,\n        lambda j: v[i][j] * x[i] * x[j]\n    )\n)\n```\n\nmathjson2qubo generates PyQUBO model from mathematical expressions formatted MathJSON (with constant values).\n\n\n## Example Usage\n\n### Install\n\n```\npip install mathjson2qubo\n```\n\n\nWe introduce the example of number partitioning.\n\n> Number partitioning is the task of deciding whether a given multiset S of positive integers can be partitioned into two subsets S1 and S2 such that the sum of the numbers in S1 equals the sum of the numbers in S2.\n\nNumber partitioning problem is formulated as follows.\n\n<img src="https://latex.codecogs.com/gif.latex?\\sum^{|S|}_in_is_i,~~n_i\\in S,~~s_i\\in\\{-1,1\\}." title="QUBO equation" />\n\n\n```python\nfrom mathjson2qubo import Parser\n\nnumbers = [3, 1, 1, 2, 2, 1]\n\nparser = Parser(\n    vartype="spin",\n    variables=[\n      {"symbol": "s", "dimension": 1, "size": len(numbers) }\n    ],\n    constants=[\n      {"symbol": "N", "values": len(numbers)},\n      {"symbol": "n", "values": numbers}\n    ]\n)\n\nmathjson = {\n    "fn": "add",\n    "arg": [\n        {"num": 0},\n        {\n            "fn": "sum",\n            "sub": {"fn": "equal", "arg": [{"sym": "i"}, {"num": 1}]},\n            "sup": {"sym": "N"},\n            "arg": [\n                {\n                    "fn": "multiply",\n                    "arg": [\n                        {"sym": "n", "sub": {"sym": "i"}},\n                        {"sym": "s", "sub": {"sym": "i"}},\n                    ],\n                }\n            ],\n        },\n    ],\n    "sup": {"num": 2},\n}\n\nobjectives = [{"label": "obj", "weight": 1, "tex": mathjson}]\n\nparser.parse_to_pyqubo_model(objectives=objectives, constraints=[])\n# > pyqubo.core.model.Model instance\n\n# Support solution by simulated annealing built in pyqubo.\nparser.solve(\n  objectives=objectives,\n  constraints=[],\n  num_reads=10,\n  sweeps=1000,\n  beta_range=(1, 50)\n)\n# > ({\'s\': {0: 1.0, 1: 1.0, 2: -1.0, 3: -1.0, 4: -1.0, 5: 1.0}}, {}, 0.0)\n\n```\n',
    'author': 'Yosuke Mukasa',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mks1412/mathjson2qubo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
